/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.conient;

//---IMPORTS---
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Collections;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * This provides a modal dialog from which the user
 * can reconfigure the conient client.
 *
 * Look at the comments for the attributes of this class to see
 * what configuration options this class deals with.
 *
 * It then writes back the configuration to the loaded configuration.
 * The user can then opt to save it.
 *
 *
 * @author  $Author: tdb $
 * @version $Id: ConfigurationDialog.java,v 1.16 2002/05/21 16:47:10 tdb Exp $
 */
public class ConfigurationDialog extends JDialog {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public static final String REVISION = "$Revision: 1.16 $";
    
//---STATIC METHODS---

//---CONSTRUCTORS---

    /**
     * Constructs and shows the dialog for the user
     */
    public ConfigurationDialog() {
        super(Conient.getFrame(), "Configuration Options", true);
        
        // setup the buttoms
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
        JButton ok = new JButton("OK");
        ok.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setNewSettingsAndClose();
            }
        });
        
        JButton cancel = new JButton("Cancel");
        cancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dispose();
            }
        });
        buttonPanel.add(Box.createHorizontalGlue());
        buttonPanel.add(ok);
        buttonPanel.add(cancel);
        

        // setup the options panes
        JTabbedPane center = new JTabbedPane();
        //center.setLayout(new BoxLayout(center, BoxLayout.Y_AXIS));
        
        // client options
        center.addTab("Client Options", createClientOptions());
        
        // _server options
        center.addTab("Server Options", createServerOptions());

        // firewall options
        center.addTab("Firewall Options", createFirewallOptions());
        
        // data options
        center.addTab("Data Options", createDataOptions());
        
        // display the current settings
        getCurrentSettings();
        
        // set the window up
        getContentPane().add(buttonPanel, "South");
        getContentPane().add(center, "Center");
        pack();
        setLocationRelativeTo(Conient.getFrame());
        setResizable(false);
        setVisible(true);
    }

//---PUBLIC METHODS---

//---PRIVATE METHODS---
   
    /**
     * creates a JPanel for the client options
     *
     * @return the build panel with the options
     */
    private JPanel createClientOptions() {
        GridBagLayout gridbag = new GridBagLayout();
        GridBagLayout gridbag2 = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.HORIZONTAL;
        
        JPanel clientPanel = new JPanel();
        clientPanel.setLayout(gridbag);
        JLabel nameLabel = new JLabel("Client name:");
        nameLabel.setHorizontalAlignment(JLabel.RIGHT);
        _name = new JTextField(20);
        JPanel nameBox = new JPanel();
        nameBox.setLayout(new GridLayout(1,2));
        nameBox.add(nameLabel);
        nameBox.add(_name);
        
        c.gridy = 0;
        gridbag.setConstraints(nameBox, c);
        clientPanel.add(nameBox);
              
        JPanel lists = new JPanel();
        lists.setLayout(gridbag2);
        
        JPanel knownHostsPanel = new JPanel();
        knownHostsPanel.setLayout(new BoxLayout(knownHostsPanel, BoxLayout.Y_AXIS));
        _hostDiscoveryMode = new JCheckBox("Discover new hosts from the server");
        knownHostsPanel.add(_hostDiscoveryMode);
        knownHostsPanel.setBorder(new TitledBorder(new LineBorder(new Color(0, 0, 102)), " Known Hosts "));
        _knownHosts = new JList();
        _knownHosts.setVisibleRowCount(10);
        JScrollPane knownHostsScrollPane = new JScrollPane(_knownHosts);
        knownHostsPanel.add(knownHostsScrollPane);
        JLabel newHostLabel = new JLabel("New host:");
        newHostLabel.setHorizontalAlignment(JLabel.RIGHT);
        _newHost = new JTextField(20);

        JButton newHostButton = new JButton("Add Host");
        newHostButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                _knownHostsData.add(_newHost.getText());
                _newHost.setText("");
                setListDisplay();
            }
        });
        JButton removeHostButton = new JButton("Remove Host(s)");
        removeHostButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Object[] selected = _knownHosts.getSelectedValues();
                for(int x = 0; x < selected.length; x++) {
                    _knownHostsData.remove(_knownHostsData.indexOf(selected[x]));
                    setListDisplay();
                }
            }
        });
        JPanel newHostBox = new JPanel();
        newHostBox.setLayout(new GridLayout(1,3));
        newHostBox.add(newHostLabel);
        newHostBox.add(_newHost);

        JPanel buttonPanel = new JPanel();
        buttonPanel.add(newHostButton);
        buttonPanel.add(removeHostButton);
        knownHostsPanel.add(buttonPanel);        
        knownHostsPanel.add(newHostBox);
        

        c.gridy = 0;
        c.gridx = 0;
        gridbag.setConstraints(knownHostsPanel, c);
        lists.add(knownHostsPanel);
                
        
        JPanel addRemoveBox = new JPanel();
        addRemoveBox.setLayout(new BoxLayout(addRemoveBox, BoxLayout.Y_AXIS));
        addRemoveBox.add(Box.createGlue());
        JButton add = new JButton(">>>");
        add.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Object[] selected = _knownHosts.getSelectedValues();
                for(int x = 0; x < selected.length; x++) {
                    _knownHostsData.remove(_knownHostsData.indexOf(selected[x]));
                    _hostListData.add(selected[x]);
                    setListDisplay();
                }
            }
        });
        
        addRemoveBox.add(add);
        addRemoveBox.add(Box.createGlue());
        JButton remove = new JButton("<<<");
        remove.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Object[] selected = _hostList.getSelectedValues();
                for(int x = 0; x < selected.length; x++) {
                    _hostListData.remove(_hostListData.indexOf(selected[x]));
                    _knownHostsData.add(selected[x]);
                    setListDisplay();
                }
            }
        });
        addRemoveBox.add(remove);
        addRemoveBox.add(Box.createGlue());
        
        c.gridy = 0;
        c.gridx = 1;
        gridbag.setConstraints(addRemoveBox, c);
        lists.add(addRemoveBox);

        JPanel hostListPanel = new JPanel();
        hostListPanel.setLayout(new BoxLayout(hostListPanel, BoxLayout.Y_AXIS));
        _useHostList = new JCheckBox("Only monitor hosts in this list");
        hostListPanel.add(_useHostList);
        hostListPanel.setBorder(new TitledBorder(new LineBorder(new Color(0, 0, 102)), " Hosts To Monitor "));
        _hostList = new JList();
        _hostList.setVisibleRowCount(10);
        JScrollPane hostListScrollPane = new JScrollPane(_hostList);
        hostListPanel.add(hostListScrollPane);
        
        c.gridy = 0;
        c.gridx = 2;
        gridbag.setConstraints(hostListPanel, c);
        lists.add(hostListPanel);
        
        c.gridy = 1;
        c.gridx = 0;
        gridbag.setConstraints(lists, c);
        clientPanel.add(lists);
        return clientPanel;
    }
    
    /**
     * creates a JPanel for the server options
     *
     * @return the build panel with the options
     */
    private JPanel createServerOptions() {
        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.HORIZONTAL; 
        JPanel serverPanel = new JPanel();
        serverPanel.setLayout(gridbag);
        _connectControl = new JCheckBox("Automatically connect the control channel");
        c.gridy = 0;
        gridbag.setConstraints(_connectControl, c);
        serverPanel.add(_connectControl);
        _connectData = new JCheckBox("Automatically connect the data channel");
        c.gridy = 1;
        gridbag.setConstraints(_connectData, c);
        serverPanel.add(_connectData);
        JLabel serverLabel = new JLabel("i-scream server:");
        serverLabel.setHorizontalAlignment(JLabel.RIGHT);
        _server = new JTextField(20);
        JPanel serverBox = new JPanel();
        serverBox.setLayout(new GridLayout(1,2));
        serverBox.add(serverLabel);
        serverBox.add(_server);
        c.gridy = 2;
        gridbag.setConstraints(serverBox, c);
        serverPanel.add(serverBox);
        JLabel portLabel = new JLabel("Client interface port:");
        portLabel.setHorizontalAlignment(JLabel.RIGHT);
        _port = new JTextField(4);
        JPanel portBox = new JPanel();
        portBox.setLayout(new GridLayout(1,2));
        portBox.add(portLabel);
        portBox.add(_port);
        c.gridy = 3;
        gridbag.setConstraints(portBox, c);
        serverPanel.add(portBox);
        return serverPanel;
    }

    /**
     * creates a JPanel for the firewall options
     *
     * @return the build panel with the options
     */
    private JPanel createFirewallOptions() {
        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.HORIZONTAL; 
        JPanel firewallPanel = new JPanel();
        firewallPanel.setLayout(gridbag);
        
        _useFirewall = new JCheckBox("Use firewall command to connect through to server");
        c.gridy = 0;
        gridbag.setConstraints(_useFirewall, c);
        firewallPanel.add(_useFirewall);

        JLabel commandLabel = new JLabel("Firewall command:");
        commandLabel.setHorizontalAlignment(JLabel.RIGHT);
        _firewallCommand = new JTextField(20);
        JPanel commandBox = new JPanel();
        commandBox.setLayout(new GridLayout(1,2));
        commandBox.add(commandLabel);
        commandBox.add(_firewallCommand);
        
        c.gridy = 1;
        gridbag.setConstraints(commandBox, c);
        firewallPanel.add(commandBox);
        
        JLabel waitLabel = new JLabel("Firewall wait time:");
        waitLabel.setHorizontalAlignment(JLabel.RIGHT);
        _firewallWait = new JTextField(2);
        JPanel waitBox = new JPanel();
        waitBox.setLayout(new GridLayout(1,2));
        waitBox.add(waitLabel);
        waitBox.add(_firewallWait);

        c.gridy = 2;        
        gridbag.setConstraints(waitBox, c);
        firewallPanel.add(waitBox);
        
        JLabel fServerLabel = new JLabel("Firewall server:");
        fServerLabel.setHorizontalAlignment(JLabel.RIGHT);
        _firewallServer = new JTextField(20);
        JPanel fServerBox = new JPanel();
        fServerBox.setLayout(new GridLayout(1,2));
        fServerBox.add(fServerLabel);
        fServerBox.add(_firewallServer);
        
        c.gridy = 3;
        gridbag.setConstraints(fServerBox, c);
        firewallPanel.add(fServerBox);
        
        return firewallPanel;
    }

    /**
     * creates a JPanel for the data options
     *
     * @return the build panel with the options
     */
    private JPanel createDataOptions() {
        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.HORIZONTAL; 
        JPanel dataPanel = new JPanel();
        dataPanel.setLayout(gridbag);
        _displayQueue = new JCheckBox("Display server queue information");
        c.gridy = 0;
        gridbag.setConstraints(_displayQueue, c);
        dataPanel.add(_displayQueue);
        _displayExtra = new JCheckBox("Display extra data found in packets");
        c.gridy = 1;
        gridbag.setConstraints(_displayExtra, c);
        dataPanel.add(_displayExtra);
        _packetDump = new JCheckBox("Dump raw packet data to the console");
        c.gridy = 2;
        gridbag.setConstraints(_packetDump, c);
        dataPanel.add(_packetDump);
        
        JLabel queueLimitLabel = new JLabel("Maximum data queue size:");
        queueLimitLabel.setHorizontalAlignment(JLabel.RIGHT);
        _queueLimit = new JTextField(4);
        JPanel queueBox = new JPanel();
        queueBox.setLayout(new GridLayout(1,2));
        queueBox.add(queueLimitLabel);
        queueBox.add(_queueLimit);
        c.gridy = 3;
        gridbag.setConstraints(queueBox, c);
        dataPanel.add(queueBox);
        return dataPanel;
    }
    
    /**
     * Reads in the current settings from the loaded configuration
     */
    private void getCurrentSettings() {
        _name.setText(_config.getProperty("clientname"));
        _server.setText(_config.getProperty("control.server"));
        _port.setText(_config.getProperty("control.port"));
        _connectControl.setSelected(_config.getProperty("control.onstartconnect").equals("1"));
        _connectData.setSelected(_config.getProperty("data.onstartconnect").equals("1"));
        _firewallCommand.setText(_config.getProperty("firewall.command"));
        _firewallCommand.moveCaretPosition(0);
        _firewallWait.setText(_config.getProperty("firewall.commandwait"));
        _firewallServer.setText(_config.getProperty("firewall.server"));
        _displayQueue.setSelected(_config.getProperty("displayQueueInformation").equals("1"));
        _displayExtra.setSelected(_config.getProperty("displayExtraData").equals("1"));
        _packetDump.setSelected(_config.getProperty("packetDump").equals("1"));
        _useFirewall.setSelected(_config.getProperty("useFirewall").equals("1"));
        _hostDiscoveryMode.setSelected(_config.getProperty("hostDiscoveryMode").equals("1"));
        _useHostList.setSelected(_config.getProperty("useHostList").equals("1"));
        _queueLimit.setText(_config.getProperty("dataQueueSize"));
        prepareLists();                
    }
    
    /**
     * Displays the host lists in the host windows
     * in alphabetical order
     */
    private void prepareLists() {
        _knownHostsData = new ArrayList();
        _hostListData = new ArrayList();
        String _configKnownHostsList = _config.getProperty("knownHostsList");
        String _configHostList = _config.getProperty("hostList");
        StringTokenizer st;
        st = new StringTokenizer(_configKnownHostsList, ";");
        while(st.hasMoreTokens()) {
            // check its not in the other list...as we only want one entry in the two
            String knownHost = st.nextToken();
            if(_configHostList.indexOf(knownHost) == -1) _knownHostsData.add(knownHost);
        }
        st = new StringTokenizer(_configHostList, ";");
        while(st.hasMoreTokens()) {
            _hostListData.add(st.nextToken());
        }
        setListDisplay();
    }
    
    /**
     * Because of the limitation of the existing
     * data model for lists, ArrayLists are used.
     * Whenever the data in these lists is changed,
     * this method is called to re-sort the lists
     * and reset the display.
     */
    private void setListDisplay() {
        // ensure the order
        Collections.sort(_hostListData);
        Collections.sort(_knownHostsData);
        
        _knownHosts.setListData(_knownHostsData.toArray());
        _hostList.setListData(_hostListData.toArray());
    }
    
    /**
     * Writes the changes back to the loaded configuration
     * Then closes the windows
     */
    private void setNewSettingsAndClose() {
        
        _config.setProperty("clientname", _name.getText().trim());
        _config.setProperty("control.server", _server.getText().trim());
        _config.setProperty("control.port", _port.getText().trim());
        _config.setProperty("firewall.command", _firewallCommand.getText().trim());
        _config.setProperty("firewall.commandwait", _firewallWait.getText().trim());
        _config.setProperty("firewall.server", _firewallServer.getText().trim());
        _config.setProperty("dataQueueSize", _queueLimit.getText().trim());
        if (_useFirewall.isSelected()) {
            _config.setProperty("useFirewall", "1");
        } else {
            _config.setProperty("useFirewall", "0");
        }
        if (_connectControl.isSelected()) {
            _config.setProperty("control.onstartconnect", "1");
        } else {
            _config.setProperty("control.onstartconnect", "0");
        }
        if (_connectData.isSelected()) {
            _config.setProperty("data.onstartconnect", "1");
        } else {
            _config.setProperty("data.onstartconnect", "0");
        }
        if (_packetDump.isSelected()) {
            _config.setProperty("packetDump", "1");
        } else {
            _config.setProperty("packetDump", "0");
        }
        if (_displayExtra.isSelected()) {
            _config.setProperty("displayExtraData", "1");
        } else {
            _config.setProperty("displayExtraData", "0");
        }
        if (_displayQueue.isSelected()) {
            _config.setProperty("displayQueueInformation", "1");
        } else {
            _config.setProperty("displayQueueInformation", "0");
        }
        if (_useHostList.isSelected()) {
            _config.setProperty("useHostList", "1");
        } else {
            _config.setProperty("useHostList", "0");
        }
        if (_hostDiscoveryMode.isSelected()) {
            _config.setProperty("hostDiscoveryMode", "1");
        } else {
            _config.setProperty("hostDiscoveryMode", "0");
        }
        
        // sort out the hosts lists
        
        // known hosts are both the known hosts list AND the host list
        // first the hostList...
        Iterator i;
        i = _hostListData.iterator();
        String _hostListSave = "";
        while(i.hasNext()) {
            _hostListSave += (String) i.next() + ";";
        }
        
        i = _knownHostsData.iterator();
        String _knownHostsListSave = "";
        while(i.hasNext()) {
            _knownHostsListSave += (String) i.next() + ";";
        }
        
        _knownHostsListSave += _hostListSave;
        _config.setProperty("hostList", _hostListSave);
        _config.setProperty("knownHostsList", _knownHostsListSave);
        dispose();
    }

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---

    /**
     * A reference to the configuration in use
     */
    Configuration _config = Configuration.getInstance();
    
    /**
     * Used to add a new host to the known host list
     */
    private JTextField _newHost;
    
    /**
     * Lists the known hosts (less the ones we're monitoring)
     */
    private JList _knownHosts;
    
    /**
     * The list of hosts we're monitoring
     */
    private JList _hostList;
    
    /**
     * If we are making a note of the hosts the server 
     * sends us.
     */
    private JCheckBox _hostDiscoveryMode;
    
    /**
     * If we are using the host list setting to only view data
     * from certain hosts
     */
    private JCheckBox _useHostList;
    
    /**
     * The name to identify this client to the server
     */
    private JTextField _name;
    
    /**
     * The _port the i-scream _server is running on
     */
    private JTextField _port;
    
    /**
     * The host_name of the i-scream server
     */
    private JTextField _server;
    
    /**
     * If we want to start the control channel on startup
     */
    private JCheckBox _connectControl;
    
    /**
     * If we want to start the data channel on startup
     */
    private JCheckBox _connectData;
    
    /**
     * If we want to use the firewall command to connect
     */
    private JCheckBox _useFirewall;
    
    /**
     * The command to run to open a pipe to the iscream server
     */
    private JTextField _firewallCommand;
    
    /**
     * How long to wait for the firewall command to execute
     */
    private JTextField _firewallWait;
    
    /**
     * The name of the machine to locally connect through to 
     * reach the i-scream server
     */
    private JTextField _firewallServer;
    
    /**
     * Whether to display server queue debugging information
     */
    private JCheckBox _displayQueue;
    
    /**
     * Whether to display extra data contained in packets
     */
    private JCheckBox _displayExtra;
    
    /**
     * Whether to dump raw packet data to the console
     */
    private JCheckBox _packetDump;
    
    /**
     * The limit on the data queue size
     */
    private JTextField _queueLimit;
    
    /**
     * holds the known hosts
     */
    private ArrayList _knownHostsData;
    
    /**
     * holds the monitored hosts
     */
    private ArrayList _hostListData;

//---STATIC ATTRIBUTES---

}
