/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.conient;

//---IMPORTS---
import uk.org.iscream.cms.util.*;
import javax.swing.*;
import java.awt.BorderLayout;
import java.net.URL;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/**
 * This panel holds the toolbar at the top.
 * The tool bar dispatches events to the ConnectionHandler.
 *
 * It also creates and starts the ConnectionHandler which is the
 * main thread for this application
 *
 * @author  $Author: tdb $
 * @version $Id: ControlPanel.java,v 1.18 2003/02/05 19:35:04 tdb Exp $
 */
public class ControlPanel extends JPanel {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.18 $";

//---STATIC METHODS---
    
//---CONSTRUCTORS---

    /**
     * Constructs a new ControlPanel.
     *
     * @param data the data panel to be passed to the ConnectionHandler
     */
    public ControlPanel(DataPanel data) {
        super();
        setLayout(new BorderLayout());
        _handler = new ConnectionHandler(data, _actionQueue);
        add(_toolBar, "North");
        _handler.start();
    }

//---PUBLIC METHODS---
   
//---PRIVATE METHODS---

    /**
     * Sets up the tool bar, adding action listeners
     * to dispatch events to the ConnectionHandler
     */
    private JToolBar setupToolBar() {
        
        JButton connectButton = new JButton("Connect", new ImageIcon(getClass().getResource("/resources/control16.gif")));
        connectButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                _actionQueue.add(new Integer(ConnectionHandler.CONNECT));
            }
        });
        JButton disconnectButton = new JButton("Disconnect", new ImageIcon(getClass().getResource("/resources/stop16.gif")));
        disconnectButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                _actionQueue.add(new Integer(ConnectionHandler.DISCONNECT));
            }
        });
        
        JButton startDataButton = new JButton("Start Data", new ImageIcon(getClass().getResource("/resources/data16.gif")));
        startDataButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                _actionQueue.add(new Integer(ConnectionHandler.STARTDATA));
            }
        });
        JButton stopDataButton = new JButton("Stop Data", new ImageIcon(getClass().getResource("/resources/stop16.gif")));
        stopDataButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                _actionQueue.add(new Integer(ConnectionHandler.STOPDATA));
            }
        });
        
        JToolBar bar = new JToolBar(JToolBar.HORIZONTAL);
        bar.add(connectButton);
        bar.add(disconnectButton);
        bar.addSeparator();
        bar.add(startDataButton);
        bar.add(stopDataButton);

        bar.add(Box.createGlue());
        JLabel image = new JLabel(new ImageIcon(getClass().getResource("/resources/i-scream.gif")));
        bar.add(image);
        return bar;
    }
    
    public JMenuBar setupMenuBar() {
        JMenuBar bar = new JMenuBar();
        
        JMenu conientMenu = new JMenu("Conient");
        JMenuItem modifiyConfig = new JMenuItem("Modifiy Configuration");
        modifiyConfig.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Configuration.getInstance().GUIReconfiguration();
            }
        });
        conientMenu.add(modifiyConfig);
        conientMenu.addSeparator();
        JMenuItem saveConfig = new JMenuItem("Save Configuration");
        saveConfig.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (Configuration.getInstance().getUsingSpecificConfig()) {
                    Configuration.getInstance().saveConfiguration();
                } else {
                    Configuration.getInstance().saveNewConfiguration();
                }
            }
        });
        conientMenu.add(saveConfig);
        
        JMenuItem saveAsConfig = new JMenuItem("Save Configuration As...");
        saveAsConfig.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Configuration.getInstance().saveNewConfiguration();
            }
        });
        conientMenu.add(saveAsConfig);
        
        JMenuItem saveDefaultConfig = new JMenuItem("Save Configuration As Default");
        saveDefaultConfig.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Configuration.getInstance().saveDefaultConfiguration();
            }
        });
        conientMenu.add(saveDefaultConfig);
        
        JMenuItem loadConfig = new JMenuItem("Load Configuration...");
        loadConfig.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Configuration.getInstance().loadConfiguration();
            }
        });
        conientMenu.add(loadConfig);
        conientMenu.addSeparator();
        JMenuItem quit = new JMenuItem("Quit");
        quit.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                _actionQueue.add(new Integer(ConnectionHandler.QUIT));
            }
        });
        conientMenu.add(quit);
        bar.add(conientMenu);
        
        JMenu connectionMenu = new JMenu("Connection");
        JMenuItem controlConnect = new JMenuItem("Connect", new ImageIcon(getClass().getResource("/resources/control16.gif")));      
        controlConnect.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                _actionQueue.add(new Integer(ConnectionHandler.CONNECT));
            }
        });
        connectionMenu.add(controlConnect);
        JMenuItem controlDisconnect = new JMenuItem("Disconnect", new ImageIcon(getClass().getResource("/resources/stop16.gif")));
        controlDisconnect.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                _actionQueue.add(new Integer(ConnectionHandler.DISCONNECT));
            }
        });
        connectionMenu.add(controlDisconnect);
        connectionMenu.addSeparator();
        JMenuItem dataConnect = new JMenuItem("Start Data", new ImageIcon(getClass().getResource("/resources/data16.gif"))); 
        dataConnect.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                _actionQueue.add(new Integer(ConnectionHandler.STARTDATA));
            }
        });
        connectionMenu.add(dataConnect);
        JMenuItem dataDisconnect = new JMenuItem("Stop Data", new ImageIcon(getClass().getResource("/resources/stop16.gif")));
        dataDisconnect.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                _actionQueue.add(new Integer(ConnectionHandler.STOPDATA));
            }
        });
        connectionMenu.add(dataDisconnect);
        bar.add(connectionMenu);
               
        return bar;
    }

//---ACCESSOR/MUTATOR METHODS---

    public JMenuBar getMenuBar() {
        return _menuBar;
    }

//---ATTRIBUTES---

    /**
     * The tool bar for the control panel
     */    
    JToolBar _toolBar = setupToolBar();
    
    /**
     * The menu bar for the Frame
     */
    JMenuBar _menuBar = setupMenuBar();
    
    /**
     * The queue that we will add actions to.
     */
    Queue _actionQueue = new Queue();
    
    /**
     * The connection handler in use by the system.
     */
    ConnectionHandler _handler;

//---STATIC ATTRIBUTES---

}
