/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.conient;

//---IMPORTS---
import uk.org.iscream.cms.util.*;
import java.util.HashMap;
import java.util.List;
import java.util.Collections;
import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.BorderFactory;
import java.awt.CardLayout;
import javax.swing.JSplitPane;
import java.util.StringTokenizer;
import javax.swing.SwingUtilities;
import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import java.awt.Dimension;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.ListSelectionModel;

/**
 * This thread reads data from the DataReader
 * it then asks the appropriate HostDisplayPanel
 * to update its data.
 *
 * @author  $Author: tdb $
 * @version $Id: DataPanel.java,v 1.30 2003/02/05 19:35:04 tdb Exp $
 */
public class DataPanel extends JSplitPane implements Runnable {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.30 $";

//---STATIC METHODS---
    
//---CONSTRUCTORS---

    /**
     * Constructs the data panel
     */
    public DataPanel() {
        super(JSplitPane.HORIZONTAL_SPLIT);

        JScrollPane left = new JScrollPane(_hostChooserList);
        JScrollPane right = new JScrollPane(_displayPane);

        setOneTouchExpandable(true);
        setLeftComponent(left);
        setRightComponent(right);
        setDividerLocation(150);
        _xmlPacketMaker = new XMLPacketMaker();
    }

//---PUBLIC METHODS---

    /**
     * Starts the DataPanel running
     */
    public void run() {
        // setup the host list we will be using
        refreshHostList();
        if(Configuration.getInstance().getProperty("displayQueueInformation").equals("1")) {
            if (_qFrame == null) {
                _qFrame = new QueueFrame();
            }
            if (!_qFrame.isVisible()) {
                _qFrame.setVisible(true);
            }
        }
        
        try {
            while(_running) {
                
                String xml = (String) _dataQueue.get(_myQueue);
                // if we want to debug the packets
                if(Configuration.getInstance().getProperty("packetDump").equals("1")) {
                    System.out.println("[PACKET DUMP]\n" + xml);
                }
                
                Conient.setQueueStatus(_dataQueue.queueSize(_myQueue), _dataQueue.elementCount());
                if (xml == null) {
                    // shouldn't really happen...but not sure
                    //_status.insert("No XML to update...",0);
                } else {
                    
                    // Use XMLPacketMaker to make an XMLPacket object.
                    
                    XMLPacket packet = _xmlPacketMaker.createXMLPacket(xml);
                    
                    String packetType = packet.getParam("packet.attributes.type");
                    if (packetType.equals("heartbeat") || packetType.equals("data")) {
                        String hostName = packet.getParam("packet.attributes.machine_name");
                        // if we're not using a fixed list
                        if(!_usingConfiguredList) {
                            // if we don't know about this host
                            // add it.
                            if(!_hostList.containsKey(hostName)) {
                                 addHostPanel(hostName);                               
                                // if we want to remember new hosts (ie, in "discovery" mode)
                                if(_config.getProperty("hostDiscoveryMode").equals("1")) {
                                    addToKnownHosts(hostName);
                                }
                            }
                        }
                        // one final check that we know about the host
                        // if we don't by now, then the server must have sent something odd
                        if(_hostList.containsKey(hostName)) {
                            if (!((HostDisplayPanel) _hostList.get(hostName)).updateHost(packet)) {
                                //throw new Exception(hostName + " sent an invalid data packet stopping data update!");
                                Conient.addMessage("WARNING{data panel}: " + hostName + " sent an invalid data or heartbeat packet");
                            }
                        } else {
                            Conient.addMessage("WARNING{data panel}: server sent data for an unexpected host - " + hostName);
                        }
                    } else if (packetType.equals("queueStat")) {
                        // check to config to see if we want queueStat packets to be processed or not
                        if(Configuration.getInstance().getProperty("displayQueueInformation").equals("1") && _qFrame != null) {
                            if (_qFrame.isVisible()) _qFrame.update(packet);
                        }
                    } else {
                        Conient.addMessage("WARNING{data panel}: and unknown packet type was received - " + packetType);
                    }
                }
            }
        } catch (Exception e) {
            Conient.addMessage("ERROR{data panel}: +" + e);
            e.printStackTrace();
        }
    }

    /**
     * This method allows other classes 
     * to shutdown this data panel.
     */
    public void shutdown() {
        _running = false;
    }

//---PRIVATE METHODS---

    /**
     * Build the host list according to the configuration.
     * If we're set to use one, it populates the display.
     * If we're using one and its empty, or if we're not using
     * one, we set that we're not so that we use all host we detect.
     *
     * See the run method for information on host discovery if we're
     * getting all the hosts.
     */
    private void refreshHostList() {
        // reset the list
        _hostList = new HashMap();
        // check we're using a set list
        if (_config.getProperty("useHostList").equals("1")) {
            // if we are get it and set up the display
            String hostList = _config.getProperty("hostList");
            if (!hostList.equals("")) {
                StringTokenizer st = new StringTokenizer(hostList, ";");
                while(st.hasMoreTokens()) {
                    String host = st.nextToken();
                    addHostPanel(host);
                }
                _usingConfiguredList = true;
            // if we've got no list setup, we set that we're taking all
            // hosts that might come in.
            } else {
                _usingConfiguredList = false;
            }
        // we're not using a list 
        } else {
            _usingConfiguredList = false;
        }
    }
    
    /**
     * Adds a new Host to the hostList and adds its
     * display to the tabbed pane.
     * Ensures the host list is in alphabetical order.
     *
     * @param host the host to add
     */
    private void addHostPanel(String host) {
        HostDisplayPanel hostPanel = new HostDisplayPanel(host);
        SwingSafeAddCard task = new SwingSafeAddCard(_displayPane, hostPanel, host);
        SwingUtilities.invokeLater(task);
        _displayPane.add(hostPanel, host);
        _hostList.put(host, hostPanel);

        // we need to reconstruct the displayed host list, so its
        // sorted.
        
        _hostChooserList.setListData((new java.util.TreeSet(_hostList.keySet())).toArray());

        Conient.addMessage("New Host added: " + host);
    }
    
    /**
     * If we are in "discovery" mode, we want to keep a list
     * of all new hosts we find, so we can add it to our list.
     * This method simply adds the host to the end of the
     * "knownHostList" property.
     *
     * @param host the host to add
     */
    private void addToKnownHosts(String host) {
        String knownHosts = _config.getProperty("knownHostsList");
        if (knownHosts.indexOf(host) == -1) {
            _config.setProperty("knownHostsList",  knownHosts + host + ";");
        }
    }
   
    /**
     * Removes all the tabs on display
     * Used to tidy up when a new data
     * channel is opened.
     */
    public void cleanUpTabs() {
        int currloc = getDividerLocation();
        _displayPaneLayout = new CardLayout();
        _displayPane = new JPanel(_displayPaneLayout);
        JScrollPane right = new JScrollPane(_displayPane);
        setRightComponent(right);
        setDividerLocation(currloc);
    }
        
//---ACCESSOR/MUTATOR METHODS---
    
    /**
     * Assigns the queue that this panel
     * will use to obtain data
     *
     * @param queue the queue
     */
    public void setQueue(Queue queue) {
        _dataQueue = queue;
        _myQueue = _dataQueue.getQueue();
    }

//---ATTRIBUTES---    

    /**
     * The state of this thread.
     */
    boolean _running = true;

    /** 
     * This card layout is used for the _displayPane
     * it allows the chooser list to select which
     * host to display
     */
    private CardLayout _displayPaneLayout = new CardLayout();
        
    /**
     * The panel where HostDisplayPanel's
     * are placed, it uses a CardLayout
     */
    private JPanel _displayPane = new JPanel(_displayPaneLayout);
    
    /**
     * The host list allows the user to select which
     * host pane is topmost
     */
    private JList _hostChooserList = new JList();
    {
        _hostChooserList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        _hostChooserList.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                int selection = _hostChooserList.getSelectedIndex();
                if (selection > -1) {
                    _displayPaneLayout.show(_displayPane, (String) _hostChooserList.getSelectedValue());
                }
            }
        });
    }
    /**
     * The queue new data will be read from
     */
    private Queue _dataQueue;
    
    /**
     * Our queue number
     */
    int _myQueue;
    
    /**
     * An icon to represent a host
     */
    private ImageIcon _serverIcon = new ImageIcon(getClass().getResource("/resources/server.gif"));
    
    /**
     * A frame to display Queue information
     * may not always be used - loaded according to config
     */
    private QueueFrame _qFrame = null; 
    
    /**
     * A reference to the configuraton object
     */
    private Configuration _config = Configuration.getInstance();
    
    /**
     * Contains a list of hosts that the data panel will use
     * or build during its operation.
     */
    private HashMap _hostList = null;

    /**
     * If we should be using a configured list or just accepting
     * all the hosts we get
     */
    boolean _usingConfiguredList = false;
    
    private XMLPacketMaker _xmlPacketMaker;
}
