/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.conient;

//---IMPORTS---
import java.io.*;
import uk.org.iscream.cms.util.*;

/**
 * The class reads in data from a BufferedReader,
 * it then converts it to an XMLPacket and adds
 * it to its Queue for anything that wants it.
 *
 * @author  $Author: tdb $
 * @version $Id: DataReader.java,v 1.17 2003/02/05 19:35:04 tdb Exp $
 */
public class DataReader extends Thread {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.17 $";
   
//---STATIC METHODS---

//---CONSTRUCTORS---

    /**
     * Constructs a new data reader, giving it its BufferedReader
     * and Queue.
     *
     * @param inBound the BufferedReader this class should use
     * @param dataQueue the queue new data should be placed on
     * @param ch the connection handler in use
     */
    public DataReader(BufferedReader inBound, Queue dataQueue, ConnectionHandler ch) {
        _ch = ch;
        _inBound = inBound;
        _dataQueue = dataQueue;
    }

//---PUBLIC METHODS---

    /**
     * This thread reads data from the BufferedReader.
     * It does this until either there is a problem
     * or it is told to stop.
     *
     * If there is a problem it calls shutdownLinks in the
     * ConnectionHandler.
     *
     * Any data it reads it converts to XML and then 
     * adds to its queue.
     */
    public void run() {
        try {
            String line;
            
            // continue until we are told to stop
            while (_running) {
                line = _inBound.readLine();
                if (line == null) {
                    throw new IOException("unexpected loss of connection to server");
                }
                _dataQueue.add(line);
            }
        
            // close the BufferedReader
            _inBound.close();
        
        } catch (IOException e) {
            Conient.addMessage("WARNING{data reader}: inbound data stopped - "+e);
            _running = false;
            // tell the ConnectionHandler to shut down the links
            _ch.shutdownDataLink();
        }
    }

    /**
     * This method allows other classes 
     * to shutdown this data reader.
     */
    public void shutdown() {
        _running = false;
    }

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---

    /**
     * The reader we are reading from.
     */
    private BufferedReader _inBound;
    
    /**
     * The Queue we place data on.
     */
    private Queue _dataQueue;
    
    /**
     * The state of this thread.
     */
    private boolean _running = true;
    
    /**
     * A reference to the ConnectionHandler in use
     */
    private ConnectionHandler _ch;
    
//---STATIC ATTRIBUTES---

}
