/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.conient;

//---IMPORTS---
import java.awt.Color;
import javax.swing.JProgressBar;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.BoxLayout;
import javax.swing.SwingUtilities;

/**
 * Shows a countdown timer for packet arrival times.
 * When constructed it takes an integer value relating
 * to the time in seconds between recieving packets.
 * 
 * This then displays a JProgress bar which updates at 
 * 10th of a second intervals.
 *
 * @author  $Author: tdb $
 * @version $Id: PacketTimer.java,v 1.8 2003/03/06 12:15:52 tdb Exp $
 */
public class PacketTimer extends JPanel implements Runnable {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public static final String REVISION = "$Revision: 1.8 $";
    
//---STATIC METHODS---

//---CONSTRUCTORS---

    /**
     * Constructs a new PacketTimer
     *
     * @param timeBetweenPackets the time in seconds between packet arrival
     */
    public PacketTimer(String name, int timeBetweenPackets) {
        _timeBetweenPackets = timeBetweenPackets * 1000;
        _progressBar = new JProgressBar(JProgressBar.HORIZONTAL, 0, _timeBetweenPackets);
        _barColour = _progressBar.getForeground();
        this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        JLabel label = new JLabel(name);
        this.add(_progressBar);
        this.add(label);
    }

//---PUBLIC METHODS---
    
    /**
     * Updates the progress bar every 10th of a second.
     */
    public void run() {
        while(_running) {
            try {
                Thread.sleep(100);
            } catch (InterruptedException e) {
            }
            
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    long t = _nextTime - System.currentTimeMillis();
                    if(t >= 0) {
                        _progressBar.setValue((int) t);
                    }
                    else {
                        _progressBar.setValue((int) (t*-1));
                        _progressBar.setForeground(Color.red);
                    }
                }
            });
        }
    }
    
    /**
     * Resets the progress bar.
     */
    public void reset() {
        _nextTime = System.currentTimeMillis() + _timeBetweenPackets;
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                _progressBar.setValue(_timeBetweenPackets);
                _progressBar.setForeground(_barColour);
            }
        });
    }
    
//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---

    /**
     * Holds the time between packets as used internally by this class
     */
    private int _timeBetweenPackets;
    private long _nextTime;
    private Color _barColour;
    
    /**
     * Used to indicate the state of the thread
     */
    private boolean _running = true;
    
    private JProgressBar _progressBar;

//---STATIC ATTRIBUTES---

}