/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.conient;

//---IMPORTS---
import javax.swing.JComponent;
import java.awt.Container;

/**
 * Allows components to be safely added to containers
 * This class takes a container and a component to add
 * to the container.  It then should be added to the 
 * Swing Event Dispatching Thread in order that the 
 * add can carry out.
 *
 * @author  $Author: tdb $
 * @version $Id: SwingSafeAdd.java,v 1.5 2002/05/21 16:47:10 tdb Exp $
 */
public class SwingSafeAdd implements Runnable {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public static final String REVISION = "$Revision: 1.5 $";
    
//---STATIC METHODS---

//---CONSTRUCTORS---

    /**
     * Constructs a new instance.
     *
     * @param container the container to add the component to
     * @param component the component to be added to the container
     */
    public SwingSafeAdd(Container container, JComponent component) {
        _container = container;
        _component = component;
    }

//---PUBLIC METHODS---

    /**
     * Executed by the Swing Event Dispatch thread
     * to update the display.
     */
    public void run() {
        _container.add(_component);
    }

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---
    
    /**
     * The component to add
     */
    private JComponent _component;
    
    /**
     * The container to add the component to
     */
    private Container _container;
    
//---STATIC ATTRIBUTES---

}