/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.conient.datacomponents;

//---IMPORTS---
import javax.swing.JPanel;
import uk.org.iscream.cms.util.XMLPacket;

/**
 * This is the abstract class for all
 * data components that will display data on the screen.
 * <br><br>
 * This is an abstract class based on a JPanel ensuring
 * that all components can be handled like a JPanel.  It
 * extends the JPanel by adding a specific setValue function
 * this tells the component to update its data display however it
 * sees fit.  This value is the string value obtained from an
 * XML packet, it is up to the component to convert and display
 * as it chooses, if there is a problem doing this then it
 * can throw a DataFormatException to indicate the problem.
 * <br><br>
 * As all extending components will be data components, it also
 * implements the DataComponent interface, which should thus be 
 * imlemented by all extending compoents.
 * <br><br>
 * All extending components should implement the run() method of
 * this class as it implements the Runnable interface.  This run
 * method should contain the code to update the actual GUI
 * component of this class.  In the setValue() method which they
 * should also be implementing, the code should add the
 * data component to the Swing Event Dispatch queue. eg,
 * <br><br>
 * SwingUtilities.invokeLater(this);
 * 
 *
 * @author  $Author: tdb $
 * @version $Id: VisibleDataComponent.java,v 1.9 2003/02/05 19:35:04 tdb Exp $
 */
public abstract class VisibleDataComponent extends JPanel implements Runnable, DataComponent {
    
//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.9 $";
    
//---STATIC METHODS---
    
//---CONSTRUCTORS---

//---PUBLIC METHODS---

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---

//---STATIC ATTRIBUTES---

}
